package ed_cr311h3;

import gui.ClosableJFrame;
import gui.run.RunMenu;
import gui.run.RunMenuItem;
import j2d.ImageUtils;
import j2d.gui.ImagePanel;

import javax.swing.*;
import javax.swing.border.EtchedBorder;
import java.awt.*;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

/**
 * Created by IntelliJ IDEA.
 * User: Cool Ed
 * Date: Feb 15, 2006
 * Time: 12:24:26 AM
 * To change this template use File | Settings | File Templates.
 */
public class ImageThumbViewer extends ClosableJFrame implements Printable {
    private int currentPage = 1;
    private int totalImages = 130;
    private int totPages = 3;
    private static final int IMGS_PER_PAGE = 40;

    private JPanel[] imgPagePanels;
    private JPanel currentImgPanel;

    private final Dimension frameSize = new Dimension(600, 770);
    private Container c;

    public ImageThumbViewer(Image[] img, String[] names) {
        super();
        totalImages = img.length;
        totPages = (totalImages / IMGS_PER_PAGE) + 1;

        this.setTitle("Thumb Viewer");
        this.setJMenuBar(createMenuBar());

        this.setSize(frameSize);
        this.setResizable(false);
        this.setPreferredSize(frameSize);

        imgPagePanels = createImgPanels(img, names);
        currentImgPanel = imgPagePanels[0];
        createPageLayout();

        this.setVisible(true);
    }

    private JMenuBar createMenuBar() {
        JMenuBar menuBar = new JMenuBar();

        menuBar.add(createFileMenu());

        menuBar.add(createViewMenu());

        return menuBar;

    }

    private RunMenu createViewMenu() {
        RunMenu view = new RunMenu("[View");

        for (int i = 0; i < totPages; i++) {
            final int tmp = i + 1;
            view.add(new RunMenuItem("Page [" + tmp) {
                private int pageNum = tmp;

                public void run() {
                    changeImgPage(pageNum);
                    System.out.println(pageNum);
                }
            });
        }
        return view;
    }

    private RunMenu createFileMenu() {
        RunMenu file = new RunMenu("[File");
        file.add(new RunMenuItem("[Print Current Page") {
            public void run() {
                printThisPage();
            }

        });
        file.add(new RunMenuItem("Print [All Pages") {
            public void run() {
                printAllPages();
            }
        });
        file.add(new RunMenuItem("[Close{control c}") {
            public void run() {
                closeFrame();
            }
        });
        return file;
    }

    private void createPageLayout() {
        c = this.getContentPane();
        c.setLayout(new BorderLayout());

        c.add(createColoredPanel(560, 70), BorderLayout.NORTH);
        c.add(createColoredPanel(70, 700), BorderLayout.WEST);
        c.add(createColoredPanel(70, 700), BorderLayout.EAST);
        c.add(createColoredPanel(560, 70), BorderLayout.SOUTH);

        c.add(currentImgPanel, BorderLayout.CENTER);
    }

    private JPanel createColoredPanel(final int w, final int h) {
        JPanel jp = new JPanel() {
            public Dimension getPreferredSize() {
                return new Dimension(w, h);
            }
        };
        jp.setSize(w, h);
        jp.setBackground(Color.WHITE);
        return jp;
    }

    private JPanel[] createImgPanels(Image[] img, String[] imgNames) {
        JPanel[] _imgPagePanels = new JPanel[totPages];
        final Font labelFont = new Font("times", Font.BOLD, 10);

        final Dimension ipSize = new Dimension(50, 50);
        final Dimension panSize = new Dimension(84, 90);

        for (int i = 0; i < totPages; i++) {
            int start = IMGS_PER_PAGE * i;
            int end;
            if (totalImages - start >= IMGS_PER_PAGE) {
                end = start + IMGS_PER_PAGE;
            } else {
                end = totalImages;
            }

            _imgPagePanels[i] = createColoredPanel(420, 560);

            for (int j = start; j < end; j++) {
                JPanel pan = new JPanel();
                pan.setLayout(new BorderLayout(0, 0));

                ImagePanel ip = new ImagePanel() {
                    public Dimension getPreferredSize() {
                        return ipSize;
                    }
                };
                ip.setImage(ImageUtils.scaleFast(img[j], 50, 50));
                ip.setPreferredSize(ipSize);

                pan.setPreferredSize(panSize);

                pan.add(ip, BorderLayout.CENTER);
                JLabel lab = new JLabel(imgNames[j]);
                System.out.println(lab.getText());
                lab.setFont(labelFont);
                pan.add(lab, BorderLayout.SOUTH);
                pan.setBackground(Color.WHITE);
                _imgPagePanels[i].add(pan);
            }

            _imgPagePanels[i].setLayout(new GridLayout(0, 5, 5, 5));
            _imgPagePanels[i].setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
        }

        return _imgPagePanels;
    }

    private void printThisPage() {
        ImageUtils.print(c, "Thumbnails page " + currentPage);
    }

    private void printAllPages() {
        int tmpPage = currentPage;
        PrinterJob printJob = PrinterJob.getPrinterJob();
        printJob.setPrintable(this);
        if (printJob.printDialog()) {
            try {
                printJob.print();
            }
            catch (PrinterException pe) {
                System.out.println("Error printing: " + pe);
            }
        }

        changeImgPage(tmpPage);
    }

    private void changeImgPage(int page) {
        if (page == 0) return;

        Container _c = this.getContentPane();
        _c.remove(currentImgPanel);

        currentPage = page - 1;
        currentImgPanel = imgPagePanels[currentPage];
        _c.add(currentImgPanel, BorderLayout.CENTER);
        currentImgPanel.revalidate();

        this.repaint();
    }

    public int print(Graphics g, PageFormat pf, int index) {
        if (index >= imgPagePanels.length) {
            return (NO_SUCH_PAGE);
        } else if (index == 0)
            return PAGE_EXISTS;//dont want to print page "0"

        // Graphics2D g2d = (Graphics2D) g;
        // g2d.translate( pf.getImageableX(), pf.getImageableY() );
        ClosableJFrame cf = new ClosableJFrame();
        cf.getContentPane().add(imgPagePanels[index]);
        cf.setSize(400, 400);
        cf.setVisible(true);
        cf.getContentPane().printAll(g);

        return PAGE_EXISTS;
    }

    public static void disableDoubleBuffering(Component c) {
        RepaintManager currentManager = RepaintManager.currentManager(c);
        currentManager.setDoubleBufferingEnabled(false);
    }

    public static void enableDoubleBuffering(Component c) {
        RepaintManager currentManager = RepaintManager.currentManager(c);
        currentManager.setDoubleBufferingEnabled(true);
    }

    private void closeFrame() {
        this.dispose();
    }
}